#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/lib/SGVector.h>
#include <shogun/evaluation/ContingencyTableEvaluation.h>
#include <shogun/labels/Labels.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/io/CSVFile.h>
#include <shogun/labels/BinaryLabels.h>
#include <shogun/classifier/AveragedPerceptron.h>
#include <shogun/lib/WrappedObjectArray.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_binary_2d_linear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/classifier_binary_2d_linear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/classifier_binary_2d_linear_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/classifier_binary_2d_linear_labels_test.dat");

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CBinaryLabels>(f_labels_train);
auto labels_test = some<CBinaryLabels>(f_labels_test);
//![create_features]

//![set_parameters]
auto learn_rate = 1.0;
auto max_iter = 1000;
auto perceptron = some<CAveragedPerceptron>(features_train, labels_train);
perceptron->set_learn_rate(learn_rate);
perceptron->set_max_iter(max_iter);
//![set_parameters]

//![train_and_apply]
perceptron->train();
perceptron->set_features(features_test);
auto labels_predict = perceptron->apply();
//![train_and_apply]

//![extract_weights]
auto weights = perceptron->get_w();
auto bias = perceptron->get_bias();
//![extract_weights]

//![evaluate_accuracy]
auto eval = some<CAccuracyMeasure>();
auto accuracy = eval->evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("averaged_perceptron.dat", 'w');
__sg_storage->append_wrapped_real(learn_rate, "learn_rate");
__sg_storage->append_wrapped_int(max_iter, "max_iter");
__sg_storage->append_wrapped_real_vector(weights, "weights");
__sg_storage->append_wrapped_real(bias, "bias");
__sg_storage->append_wrapped_real(accuracy, "accuracy");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
