#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/io/CSVFile.h>
#include <shogun/labels/BinaryLabels.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/classifier/svm/LibSVM.h>
#include <shogun/kernel/GaussianKernel.h>
#include <shogun/lib/SGVector.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/evaluation/ContingencyTableEvaluation.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_binary_2d_nonlinear_features_train.dat");
auto f_feats_test = some<CCSVFile>("../../data/classifier_binary_2d_nonlinear_features_test.dat");
auto f_labels_train = some<CCSVFile>("../../data/classifier_binary_2d_nonlinear_labels_train.dat");
auto f_labels_test = some<CCSVFile>("../../data/classifier_binary_2d_nonlinear_labels_test.dat");

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
auto features_test = some<CDenseFeatures<float64_t>>(f_feats_test);
auto labels_train = some<CBinaryLabels>(f_labels_train);
auto labels_test = some<CBinaryLabels>(f_labels_test);
//![create_features]

//![set_parameters]
auto C = 1.0;
auto epsilon = 0.001;
auto gauss_kernel = some<CGaussianKernel>(features_train, features_train, 15);
//![set_parameters]

//![create_instance]
auto svm = some<CLibSVM>(C, gauss_kernel, labels_train);
svm->set_epsilon(epsilon);
//![create_instance]

//![train_and_apply]
svm->train();
auto labels_predict = svm->apply_binary(features_test);
//![train_and_apply]

//![extract_weights_bias]
auto alphas = svm->get_alphas();
auto b = svm->get_bias();
//![extract_weights_bias]

//![evaluate_accuracy]
auto eval = some<CAccuracyMeasure>();
auto accuracy = eval->evaluate(labels_predict, labels_test);
//![evaluate_accuracy]

// additional integration testing variables
auto output = labels_predict->get_labels();

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("kernel_svm.dat", 'w');
__sg_storage->append_wrapped_real(C, "C");
__sg_storage->append_wrapped_real(epsilon, "epsilon");
__sg_storage->append_wrapped_real_vector(alphas, "alphas");
__sg_storage->append_wrapped_real(b, "b");
__sg_storage->append_wrapped_real(accuracy, "accuracy");
__sg_storage->append_wrapped_real_vector(output, "output");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
