#include <shogun/base/init.h>
#include <shogun/base/some.h>
#include <shogun/distance/EuclideanDistance.h>
#include <shogun/lib/SGVector.h>
#include <shogun/lib/SGMatrix.h>
#include <shogun/io/CSVFile.h>
#include <shogun/lib/WrappedObjectArray.h>
#include <shogun/features/DenseFeatures.h>
#include <shogun/io/SerializableAsciiFile.h>
#include <shogun/clustering/Hierarchical.h>

using namespace shogun;

int main(int, char*[])
{
init_shogun_with_defaults();

auto f_feats_train = some<CCSVFile>("../../data/classifier_4class_2d_linear_features_train.dat");

//![create_features]
auto features_train = some<CDenseFeatures<float64_t>>(f_feats_train);
//![create_features]

//![choose_distance]
auto distance = some<CEuclideanDistance>(features_train, features_train);
//![choose_distance]

//![create_instance]
auto merges = 3;
auto hierarchical = some<CHierarchical>(merges, distance);
//![create_instance]

//![train_model]
hierarchical->train();
//![train_model]

//![extract_results]
auto d = hierarchical->get_merge_distances();
auto cp = hierarchical->get_cluster_pairs();
//![extract_results]

// Serialize output for integration testing (automatically generated)
auto __sg_storage = some<CWrappedObjectArray>();
auto __sg_storage_file = some<CSerializableAsciiFile>("hierarchical.dat", 'w');
__sg_storage->append_wrapped_int(merges, "merges");
__sg_storage->append_wrapped_real_vector(d, "d");
__sg_storage->save_serializable(__sg_storage_file);

exit_shogun();
return 0;
}
